


/****************************************************************************
;
; NAME: ostransport.h
;
;
; DESCRIPTION:
;
;       Linux transport header
;
;
; REVISION HISTORY:
;
;       $Log: /EsasHba2/Linux/driver/ostransport.h $
; 
; 11    5/30/13 9:05a Pcullen
; Another modification of the device listing in our proc file for Hitachi
; (IR 8703)
; 
; 10    1/17/13 3:23p Pcullen
; IR 8307: Change order of devices in driver proc file.
; 
; 9     1/09/12 4:24p Pcullen
; IR7845	Kernel Panic in CentOS 5.3
; IR7864	Need to be able to report enclosure IDs and slot information in
; SAS transport layer.
; NOTE: ostransport.[ch] were branched to use the new discovery code in
; the HW layer. That code is only in the ESASHBA2 HW layer at this point,
; and since ostransport is common between ESASHBA2 and ESASHBA3, it had
; to be branched. The branch should be merged once the ESASHBA3 HW layer
; contains the same discovery functionality.
; 
; 7     11/10/11 10:45a Jseba
; Add PHY discovery retries
; 
; 6     9/16/11 3:01p Jseba
; Use SAS ports for SLES 10 SP 3+
; 
; 5     7/28/11 4:48p Jseba
; Support for ESAS3HBA
; 
; 4     5/19/10 1:23p Pcullen
; Remove usage of bool, true and false.
; 
; 3     4/22/10 8:16a Pcullen
; Implementation of Linux SAS Transport
; 
; 2     11/24/09 11:05a Pcullen
; Initial checkin from VMware 4.0 porting
; 
; 1     11/20/09 4:19p Pcullen
; Rudimentary Linux SAS Transport implementation
; 
; 1     8/06/09 1:33p Pcullen
; Provide more complete implementation of Linux FC Transport, including
; remote port api (rports)
; 
;
;
; THIS PROGRAM AND THE INFORMATION CONTAINED HEREIN IS THE PROPERTY OF
; ATTO TECHNOLOGY, INC. AND SHALL NOT BE REPRODUCED, COPIED, OR USED IN
; WHOLE OR IN PART OTHER THAN AS PROVIDED FOR IN THE LICENSE AGREEMENT
; PURSUANT TO WHICH IT WAS FURNISHED.
;
; COPYRIGHT (c) ATTO TECHNOLOGY, INC. 2009
; ALL RIGHTS RESERVED.
;
;***************************************************************************/

#ifndef _OSTRANSPORT_H_
#define _OSTRANSPORT_H_

#ifndef ATTO_USE_TRANSPORT
#error Including ostransport.h and ATTO_USE_TRANSPORT not defined
#endif

/* We require at least kernel 2.6.18 to use the Linux SAS Transport */
/* Note that SLES 10 SP 3 & 4 (based on 2.6.16) have forward ported */
/* the required features, so we can use SAS ports on them.          */
#if defined (ATTO_USE_SAS_PORTS) && (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18) || defined (ATTO_SLES10SP34))
#include <linux/list.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_transport.h>

#define ATTO_TRANSPORT

/* The Linux Transport entry points. */
struct scsi_transport_template *atto_get_transport_template (void);
int atto_attach_transport (void);
void atto_release_transport (void);
void atto_init_host_attr (struct Scsi_Host *sh, int index);


#include <scsi/scsi_transport_sas.h>
#define ATTO_SAS_PORTS

#if defined (ESAS2HBA) || defined (ESAS3HBA)
#define ATTO_MAX_SAS_LINK_RATE   SAS_LINK_RATE_6_0_GBPS
#else
#define ATTO_MAX_SAS_LINK_RATE   SAS_LINK_RATE_3_0_GBPS
#endif

#define ATTO_PHY_DISCOVERY_RETRIES   5
#define ATTO_PHY_DISCOVERY_DELAY_MS  20

/* The following types and functions are used to implement the         */
/* SAS Topology, which is exposed via sysfs and the SAS Transport API. */


typedef struct Esas_Device ESASDEVICE, *PESASDEVICE;
typedef struct Esas_Port ESASPORT, *PESASPORT;
typedef struct Esas_Topology ESASTOPOLOGY, *PESASTOPOLOGY;

struct Esas_Device {
    struct list_head     list;            /* List of devices in topo.       */
    struct device        *dev;            /* Linux device pointer           */
    u64                  sas_address;     /* SAS Address of this device     */
    PESASDEVICE          parent_device;   /* pointer to control device      */
    u8                   num_phys;        /* Number of Phys in this device  */
    struct sas_phy       **phy_list;      /* array of pointers to sas phys  */
    struct list_head     port_list;       /* list of ports on this device   */
    u16                  target_id;       /* HW target ID                   */
    struct sas_rphy      *rphy;           /* Device remote phy              */
    PESASPORT            parent_port;     /* Backlinked port (exp only)     */
    u16                  flags;           /* Device flags - see below       */
    #define EDF_IS_HOST            0x0001 /* Device is host                 */
    #define EDF_IS_TARGET          0x0002 /* Device is target               */
    #define EDF_IS_EDGE_EXPANDER   0x0010 /* Device is edge expander        */
    #define EDF_IS_FANOUT_EXPANDER 0x0020 /* Device is fanout expander      */
    #define EDF_IS_ACTIVE          0x0100 /* Device is active               */

    #define EDF_IS_EXPANDER (EDF_IS_EDGE_EXPANDER | EDF_IS_FANOUT_EXPANDER)
};

struct Esas_Port {
    struct list_head  list;        /* List of ports                         */
    PESASDEVICE       near_device; /* Device in port closer to host         */
    PESASDEVICE       far_device;  /* Device in port further from host      */
    struct sas_port   *port;       /* Pointer to SAS port.                  */
};

struct Esas_Topology {
    u32              host_no;            /* Linux host number               */
    struct list_head sas_device_list;    /* List head of all device in topo */
    struct mutex     sas_topology_mutex; /* Mutex for accessing this topo   */
    PESASDEVICE      host_device;        /* The host device in this topo    */
    int              phy_index;          /* Global phy index for this topo  */
    u8               status;             /* Status of this topo             */
    #define ET_STATUS_NONE      0x00     /* Uninitialized                   */
    #define ET_STATUS_INIT      0x01     /* Initialized but not discovered  */
    #define ET_STATUS_RUNNING   0x02     /* Discovered and active           */
    #define ET_STATUS_UNLOADING 0x03     /* Driver unloading                */
};

/* ESAS Port functions */
PESASPORT esas_port_create (void);
void esas_port_destroy (PESASPORT eport);
int esas_port_add_host_phys (PESASTOPOLOGY etop, PESASPORT eport);
int esas_port_add_expander_phys (PESASTOPOLOGY etop, PESASPORT eport);
int esas_port_get_near_phy_info (PESASTOPOLOGY etop,
                                 PESASPORT eport,
                                 PESAS_SMP_DISCOVER psd);
PESASPORT esas_port_by_sas_address (PESASDEVICE edev,
                                    u64 sas_address);


/* ESAS Device functions */
PESASDEVICE esas_device_create (u64 sas_address);
void esas_device_destroy (PESASDEVICE edevice);
void esas_device_set_expander_type (PESASTOPOLOGY etop,
                                    PESASDEVICE edev);
int esas_device_add_expander_phys (PESASTOPOLOGY etop, PESASDEVICE edev);
static inline u8 esas_device_is_host (PESASDEVICE edev)
{
    return (edev->flags & EDF_IS_HOST);
}

int esas_device_make_expander (PESASTOPOLOGY etop, PESASPORT eport,
                               PESASDEVICE this_device);
int esas_device_make_end_device (PESASTOPOLOGY etop,
                                 PESASPORT eport,
                                 PESASDEVICE this_device);
int esas_device_add_parent_port (PESASTOPOLOGY etop, PESASDEVICE this_device);
void esas_device_remove_parent_port (PESASDEVICE this_device);

/* ESAS Topology functions */
void esas_topology_add_device (PESASTOPOLOGY etopology,
                                      u16 target_id, u8 device_type);
void esas_topology_remove_device (PESASTOPOLOGY etopology,
                                 u16 target_id);
void esas_topology_deactivate_device (PESASTOPOLOGY etopology,
                                     u16 target_id);
void esas_topology_activate_device (PESASTOPOLOGY etopology,
                                   u16 target_id);
void esas_topology_scan_device (PESASTOPOLOGY etopology,
                                u16 target_id);
void esas_topology_validate_device (PESASTOPOLOGY etopology,
                                    u16 target_id);
int esas_topology_list_devices (PESASTOPOLOGY etopology, char *buffer);
int esas_topology_get_device_relative_order (PESASTOPOLOGY etopology, int target_id);
void esas_topology_destroy (PESASTOPOLOGY etopology);
PESASTOPOLOGY esas_topology_by_host_no (u32 host_no);
PESASDEVICE esas_device_by_address (PESASTOPOLOGY t,
                                    u64 a);
PESASDEVICE esas_device_by_target_id (PESASTOPOLOGY t,
                                      u16 target_id);
u8 esas_device_is_valid (PESASTOPOLOGY etop, PESASDEVICE edev);
u8 esas_device_is_active (PESASDEVICE edev);
void esas_device_set_active (PESASDEVICE edev);
void esas_device_set_inactive (PESASDEVICE edev);


/* ESAS Transport Helper functions */
enum sas_protocol atto_proto_to_sas_proto (u32 atto_proto,
                                           u8 is_target);
enum sas_linkrate atto_linkrate_to_sas_linkrate (u8 atto_rate);


extern ESASTOPOLOGY esas_topology[];



#else /* defined (ATTO_USE_SAS_PORTS) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18) */

#define atto_get_transport_template() NULL
#define atto_attach_transport() (0)
#define atto_release_transport()
#define atto_init_host_attr(_sh, index)

#endif /* defined (ATTO_USE_SAS_PORTS) && LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18) */

#endif /* _OSTRANSPORT_H_ */
